program mandelf

use pca_futils
implicit none

!======================================================================!

integer,parameter               :: npix=800,maxiter=1000
real(kind=8)                    :: xmin=-2.0d0,xmax=2.0d0,ymin=-2.0d0,ymax=2.0d0
integer                         :: tt
character                       :: pressed=' '
real,dimension(:,:),allocatable :: map
allocate(map(npix,npix))
do while ( (pressed.NE.'x') .AND. (pressed.NE.'q') ) 
   call tick(tt)
   call make_mandel_map(map,xmin,xmax,ymin,ymax,npix,maxiter)
   call tock(tt)
   pressed=display_map(map,npix,-123456.0,-123456.0,0.3,xmin,xmax,ymin,ymax)
end do
deallocate(map)

!======================================================================!

contains

!----------------------------------------------------------------------!

subroutine my_pgctab(r,Re,g,Ge,b,Be,nc)
  implicit none
  real,    intent(in) :: r,Re,g,Ge,b,Be
  integer, intent(in) :: nc
  integer             :: i
  real,dimension(nc)  :: rr,gg,bb,ll
  do i = 1,nc
     rr(i) = r+(Re-r)*(i-1.0)/(1.0*(nc-1))
     gg(i) = g+(Ge-g)*(i-1.0)/(1.0*(nc-1))
     bb(i) = b+(Be-b)*(i-1.0)/(1.0*(nc-1))
     ll(i) = (i-1.0)/(1.0*(nc-1))
  enddo
  call pgctab(ll,rr,gg,bb,nc,1.0,0.5)
end subroutine my_pgctab

!----------------------------------------------------------------------!

character function display_map(map,npix,fmin,fmax,ind,xmin,xmax,ymin,ymax)
  implicit none
  real,dimension(npix,npix), intent(in)    :: map
  integer,                   intent(in)    :: npix
  real,                      intent(in)    :: fmin,fmax
  real,                      intent(in)    :: ind
  real(kind=8),              intent(inout) :: xmin,xmax,ymin,ymax
  !
  real,dimension(npix,npix) :: map_copy
  real,dimension(6)         :: tr = (/ 0.0, 1.0, 0.0, 0.0, 0.0, 1.0 /)
  integer                   :: i,j
  real                      :: myfmin,myfmax
  character                 :: press
  real                      :: fx1in,fy1in,fx2in,fy2in
  real(kind=8)              :: dfx1in,dfx2in,dfy1in,dfy2in,dx,dy,xcent,ycent
  logical,save              :: firsttime=.TRUE.
  if (firsttime) then
     call pgopen("/xwindow")
     call pgenv(1.0,1.0*npix,1.0,1.0*npix,1,-2)
     call my_pgctab(0.0,1.0,0.0,1.0,1.0,0.0,256)
     firsttime = .FALSE.
  end if
  if (fmin.EQ.-123456.0) then
     myfmin = minval(map)
  else
     myfmin = fmin
  end if
  if (fmax.EQ.-123456.0) then
     myfmax = maxval(map)
  else
     myfmax = fmax
  end if
  do i = 1,npix
     do j = 1,npix
        map_copy(i,j) = map(i,j)**ind
     end do
  end do
  myfmax = myfmax**ind
  myfmin = myfmin**ind
  call pgimag(map_copy,npix,npix,1,npix,1,npix,myfmin,myfmax,tr) 
  call pgcurs(fx1in,fy1in,press)
  if ( (press.EQ.'x') .or. (press.EQ.'q') ) then
     xmin = (fx1in/npix)*(xmax-xmin)+xmin;
     ymin = (fy1in/npix)*(ymax-ymin)+ymin;
     display_map = press
     return
  end if
  call pgsci(3)
  call pgband(2,0,fx1in,fy1in,fx2in,fy2in,press)
  dfx1in = (fx1in/npix)*(xmax-xmin)+xmin
  dfx2in = (fx2in/npix)*(xmax-xmin)+xmin
  dfy1in = (fy1in/npix)*(ymax-ymin)+ymin
  dfy2in = (fy2in/npix)*(ymax-ymin)+ymin
  dx = abs(dfx2in-dfx1in)
  dy = abs(dfy2in-dfy1in)
  if (dy>dx) then
     dx = dy
  end if
  xcent = 0.5d0*(dfx1in+dfx2in)
  ycent = 0.5d0*(dfy1in+dfy2in)
  xmin = xcent-0.5d0*dx
  xmax = xmin+dx
  ymin = ycent-0.5d0*dx
  ymax = ymin+dx
  print *,"limits are", xmin, xmax, ymin, ymax
  display_map = press
end function display_map

!----------------------------------------------------------------------!

integer function how_many_iter(a,maxiter)
  implicit none
  complex(kind=8), intent(in)  :: a
  integer,         intent(in)  :: maxiter
  integer          :: i
  complex(kind=8)  :: b
  real(kind=8)     :: r
  i = 0;
  b = a;
  how_many_iter=maxiter
  do while (i.LT.maxiter) 
     r = real(b*conjg(b))
     if (r.GT.4) then
        how_many_iter = i
        return
     end if
     b = b*b+a;
     i = i+1
  end do
end function how_many_iter

!----------------------------------------------------------------------!

subroutine make_mandel_map(mymap,xmin,xmax,ymin,ymax,npix,maxiter)
  implicit none
  real,dimension(npix,npix), intent(out) :: mymap
  real(kind=8),              intent(in)  :: xmin,xmax,ymin,ymax
  integer,                   intent(in)  :: npix,maxiter
  real(kind=8) :: x,y
  integer      :: i,j,k
  do i = 1,npix
     do j = 1,npix
        x = (1.0*i)/(1.0*npix)*(xmax-xmin)+xmin
        y = (1.0*j)/(1.0*npix)*(ymax-ymin)+ymin
        k = how_many_iter(complex(x,y),maxiter)
        mymap(i,j) = 1.0*k
     end do
  end do
end subroutine make_mandel_map

!----------------------------------------------------------------------!

end program mandelf
