#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <cpgplot.h>
#include <complex.h>
#include "pca_utils.h"

#define AUTOSCALE -123456

void make_mandel_map(float** map,double xmin,double xmax,double ymin,double ymax,int npix,int maxiter);
char display_map(float** map,int npix,int fmin,int fmax,double ind,double* xmin,double* xmax,double* ymin,double* ymax);

/*======================================================================*/

int main(int argc, char *argv[])
{
  const int npix    =  800;
  const int maxiter = 1000;
  double    xmin = -2.0;
  double    xmax =  2.0;
  double    ymin = -2.0;
  double    ymax =  2.0;
  pca_time  tt;
  char      pressed=' ';
  float**   map;
  map=fmatrix(npix,npix);
  while (pressed!='x'&&pressed!='q') {
    tick(&tt);
    make_mandel_map(map,xmin, xmax,ymin,ymax,npix,maxiter);
    tock(&tt);
    pressed=display_map(map,npix,AUTOSCALE,AUTOSCALE,0.3,&xmin,&xmax,&ymin,&ymax);
  }
  free_fmatrix(map);
}

/*======================================================================*/

void my_pgctab(float r,float R, float g, float G, float b, float B, int nc)
{
  float *rr = (float*)malloc(sizeof(float)*nc);
  float *gg = (float*)malloc(sizeof(float)*nc);
  float *bb = (float*)malloc(sizeof(float)*nc);
  float *ll = (float*)malloc(sizeof(float)*nc);
  for (int i=0;i<nc;i++) {
    rr[i]=r+(R-r)*(float)i/(float)(nc-1);
    gg[i]=g+(G-g)*(float)i/(float)(nc-1);
    bb[i]=b+(B-b)*(float)i/(float)(nc-1);
    ll[i]=(float)i/(float)(nc-1);
  }
  cpgctab(ll,rr,gg,bb,nc,1.0,0.5);
  free(rr);
  free(gg);
  free(bb);
  free(ll); 
}

/*----------------------------------------------------------------------*/

char display_map(float **map, int npix, int fmin, int fmax, double ind, 
                 double *xmin, double *xmax, double *ymin, double *ymax)
{ 
  static int firsttime=1;
  if (firsttime) {
    cpgopen("/xwindow");
    cpgenv(1,npix,1,npix,1,-2);  
    my_pgctab(0,1,0,1,1,0,256);
    firsttime=0;
  }
  float tr[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
  if (fmin==AUTOSCALE){
    fmin=map[0][0];
    for (int i=0;i<npix;i++)
      for (int j=0;j<npix;j++)
	if (map[i][j]<fmin)
	  fmin=map[i][j];
  } 
  if (fmax==AUTOSCALE){
    fmax=map[0][0];
    for (int i=0;i<npix;i++)
      for (int j=0;j<npix;j++)
	if (map[i][j]>fmax)
	  fmax=map[i][j];
  }
  float **map_copy=fmatrix(npix,npix);
  for (int i=0;i<npix;i++)
    for (int j=0;j<npix;j++)
      map_copy[i][j]=pow(map[j][i],ind); //take transpose for plotting purposes
  fmin=pow(fmin,ind);
  fmax=pow(fmax,ind);
  cpgimag(map_copy[0],npix,npix,1,npix,1,npix,fmin,fmax,tr);
  free(map_copy[0]);
  free(map_copy);
  float fx1in, fy1in, fx2in, fy2in;
  char press;
  cpgcurs(&fx1in,&fy1in,&press);
  if (press=='x'||press=='q') {
    *xmin=(fx1in/npix)*(*xmax-*xmin)+(*xmin);
    *ymin=(fy1in/npix)*(*ymax-*ymin)+(*ymin);
    return press;
  }
  cpgsci(3);  
  cpgband(2,0,fx1in,fy1in,&fx2in,&fy2in,&press);
  fprintf(stderr,"limits are %10.3f %10.3f, %10.3f %10.3f\n",fx1in,fx2in,fy1in,fy2in);
  double dfx1in=(fx1in/npix)*(*xmax-*xmin)+(*xmin);
  double dfx2in=(fx2in/npix)*(*xmax-*xmin)+(*xmin);
  double dfy1in=(fy1in/npix)*(*ymax-*ymin)+(*ymin);
  double dfy2in=(fy2in/npix)*(*ymax-*ymin)+(*ymin);
  double dx=fabs(dfx2in-dfx1in);
  double dy=fabs(dfy2in-dfy1in);
  if (dy>dx)
    dx=dy;
  double xcent=0.5*(dfx1in+dfx2in);
  double ycent=0.5*(dfy1in+dfy2in);
  *xmin=xcent-0.5*dx;
  *xmax=*xmin+dx;
  *ymin=ycent-0.5*dx;
  *ymax=*ymin+dx;
  return press;
}

/*----------------------------------------------------------------------*/

int how_many_iter(double complex a, int maxiter)
{
  int            i=0;
  double complex b=a;
  double         r;
  while (i<maxiter) {
    r=cimag(b)*cimag(b)+creal(b)*creal(b);
    if (r>4)
      return i;
    b=b*b+a;
    i++;
  }
  return maxiter;
}

/*----------------------------------------------------------------------*/

void make_mandel_map(float** map, double xmin, double xmax, double ymin, double ymax, int npix, int maxiter)
{
  for (int i=0;i<npix;i++)  
    for (int j=0;j<npix;j++) {
      double x=((double)i)/((double)npix)*(xmax-xmin)+xmin;
      double y=((double)j)/((double)npix)*(ymax-ymin)+ymin;
      double complex a=x+I*y;
      map[i][j]=how_many_iter(a,maxiter);
    }
}

/*----------------------------------------------------------------------*/
